#include <boost/foreach.hpp>

#include "AstModDataStructures.h"

//ActiveCallsExtension BEGIN
void ActiveCallsExtension::unserialize(boost::property_tree::ptree &pt)
{
	this->telNum = pt.get<std::string > ("telNum");
	this->extensionTypeId = pt.get<int > ("extensionTypeId");
}

boost::property_tree::ptree ActiveCallsExtension::serialize()
{
	boost::property_tree::ptree pt;
	pt.put("telNum", this->telNum);
	pt.put("extensionTypeId", this->extensionTypeId);
	return pt;
}
//ActiveCallsExtension END


//CallConstraintsRule BEGIN
void CallConstraintsRule::unserialize(boost::property_tree::ptree &pt)
{
	this->id = pt.get<int > ("id");
	this->name = pt.get<std::string > ("name");
	this->callCountEnabled = pt.get<bool > ("callCountEnabled");
	this->callCount = pt.get<int > ("callCount");
	this->callCountPeriod = pt.get<int > ("callCountPeriod");
	this->callCountActive = pt.get<bool > ("callCountActive");
	this->callDurationEnabled = pt.get<bool > ("callDurationEnabled");
	this->callDuration = pt.get<int > ("callDuration");
	this->callDurationPeriod = pt.get<int > ("callDurationPeriod");
	this->callDurationActive = pt.get<bool > ("callDurationActive");
	this->actionBlock = pt.get<bool > ("actionBlock");
	this->actionEmail = pt.get<bool > ("actionEmail");
	this->applyToType = pt.get<int > ("applyToType");
	BOOST_FOREACH(boost::property_tree::ptree::value_type &val, pt.get_child("prefixes"))
	{
		this->prefixes.push_back(val.second.data());
	}

}

boost::property_tree::ptree CallConstraintsRule::serialize()
{
	boost::property_tree::ptree pt;
	pt.put("id", this->id);
	pt.put("name", this->name);
	pt.put("callCountEnabled", this->callCountEnabled);
	pt.put("callCount", this->callCount);
	pt.put("callCountPeriod", this->callCountPeriod);
	pt.put("callCountActive", this->callCountActive);
	pt.put("callDurationEnabled", this->callDurationEnabled);
	pt.put("callDuration", this->callDuration);
	pt.put("callDurationPeriod", this->callDurationPeriod);
	pt.put("callDurationActive", this->callDurationActive);
	pt.put("actionBlock", this->actionBlock);
	pt.put("actionEmail", this->actionEmail);
	pt.put("applyToType", this->applyToType);
	boost::property_tree::ptree ptItems;
	BOOST_FOREACH(std::string item, this->prefixes)
	{
		ptItems.push_back(std::make_pair("", item));
	}
	pt.add_child("prefixes", ptItems);
	return pt;
}
//CallConstraintsRule END


//CallPermissionsDisabledExtension BEGIN
void CallPermissionsDisabledExtension::unserialize(boost::property_tree::ptree &pt)
{
	this->username = pt.get<std::string > ("username");
	this->telNum = pt.get<std::string > ("telNum");
	this->disabled = pt.get<bool > ("disabled");
}

boost::property_tree::ptree CallPermissionsDisabledExtension::serialize()
{
	boost::property_tree::ptree pt;
	pt.put("username", this->username);
	pt.put("telNum", this->telNum);
	pt.put("disabled", this->disabled);
	return pt;
}
//CallPermissionsDisabledExtension END


//CallPermissionsEntry BEGIN
void CallPermissionsEntry::unserialize(boost::property_tree::ptree &pt)
{
	this->groupId = pt.get<int > ("groupId");
	this->policy = pt.get<int > ("policy");
	this->prefix = pt.get<std::string > ("prefix");
	this->timeRangeName = pt.get<std::string > ("timeRangeName");
	this->groupName = pt.get<std::string > ("groupName");
}

boost::property_tree::ptree CallPermissionsEntry::serialize()
{
	boost::property_tree::ptree pt;
	pt.put("groupId", this->groupId);
	pt.put("policy", this->policy);
	pt.put("prefix", this->prefix);
	pt.put("timeRangeName", this->timeRangeName);
	pt.put("groupName", this->groupName);
	return pt;
}
//CallPermissionsEntry END


//CallPermissionsExtension BEGIN
void CallPermissionsExtension::unserialize(boost::property_tree::ptree &pt)
{
	this->telNum = pt.get<std::string > ("telNum");
	this->groupId = pt.get<int > ("groupId");
}

boost::property_tree::ptree CallPermissionsExtension::serialize()
{
	boost::property_tree::ptree pt;
	pt.put("telNum", this->telNum);
	pt.put("groupId", this->groupId);
	return pt;
}
//CallPermissionsExtension END


//CallPermissionsInterface BEGIN
void CallPermissionsInterface::unserialize(boost::property_tree::ptree &pt)
{
	this->name = pt.get<std::string > ("name");
	this->groupId = pt.get<int > ("groupId");
}

boost::property_tree::ptree CallPermissionsInterface::serialize()
{
	boost::property_tree::ptree pt;
	pt.put("name", this->name);
	pt.put("groupId", this->groupId);
	return pt;
}
//CallPermissionsInterface END


//DialplanExtension BEGIN
void DialplanExtension::unserialize(boost::property_tree::ptree &pt)
{
	this->telNum = pt.get<std::string > ("telNum");
	this->type = pt.get<int > ("type");
}

boost::property_tree::ptree DialplanExtension::serialize()
{
	boost::property_tree::ptree pt;
	pt.put("telNum", this->telNum);
	pt.put("type", this->type);
	return pt;
}
//DialplanExtension END


//TimeRangeEntry BEGIN
void TimeRangeEntry::unserialize(boost::property_tree::ptree &pt)
{
	this->groupId = pt.get<int > ("groupId");
	this->groupName = pt.get<std::string > ("groupName");
	this->entryEnabled = pt.get<int > ("entryEnabled");
	this->entryDescription = pt.get<std::string > ("entryDescription");
	this->entryType = pt.get<int > ("entryType");
	std::string _entryDays = pt.get<std::string > ("entryDays");
	strncpy(this->entryDays, _entryDays.c_str(), sizeof (this->entryDays));
	this->entryStart = pt.get<int > ("entryStart");
	this->entryEnd = pt.get<int > ("entryEnd");
}

boost::property_tree::ptree TimeRangeEntry::serialize()
{
	boost::property_tree::ptree pt;
	pt.put("groupId", this->groupId);
	pt.put("groupName", this->groupName);
	pt.put("entryEnabled", this->entryEnabled);
	pt.put("entryDescription", this->entryDescription);
	pt.put("entryType", this->entryType);
	pt.put("entryDays", this->entryDays);
	pt.put("entryStart", this->entryStart);
	pt.put("entryEnd", this->entryEnd);
	return pt;
}
//TimeRangeEntry END


//RoutingTableItem BEGIN
void RoutingTableItem::unserialize(boost::property_tree::ptree &pt)
{
	this->id = pt.get<int > ("id");
	this->internal = pt.get<std::string > ("internal");
	this->external = pt.get<std::string > ("external");
	this->iface = pt.get<std::string > ("iface");
	this->hideCallerId = pt.get<bool > ("hideCallerId");
}

boost::property_tree::ptree RoutingTableItem::serialize()
{
	boost::property_tree::ptree pt;
	pt.put("id", this->id);
	pt.put("internal", this->internal);
	pt.put("external", this->external);
	pt.put("iface", this->iface);
	pt.put("hideCallerId", this->hideCallerId);
	return pt;
}
//RoutingTableItem END
