#ifndef AST_DATA_STRUCTURES_H
#define	AST_DATA_STRUCTURES_H
#include <stdio.h>
#include <string>
#include <vector>
#include <boost/property_tree/ptree.hpp>

class AstDataStructureBase {
public:

	AstDataStructureBase() { };

	virtual ~AstDataStructureBase() { };

	virtual void unserialize(boost::property_tree::ptree &pt) = 0;
	virtual boost::property_tree::ptree serialize() = 0;
};


class ActiveCallsExtension : public AstDataStructureBase {
public:
	std::string telNum;
	int extensionTypeId;

	ActiveCallsExtension() :
				extensionTypeId(0)
	{
	}

	ActiveCallsExtension(boost::property_tree::ptree &pt) {
		this->unserialize(pt);
	}
	void unserialize(boost::property_tree::ptree &pt);
	boost::property_tree::ptree serialize();
};

class CallConstraintsRule : public AstDataStructureBase {
public:
	int id;
	std::string name;
	bool callCountEnabled;
	int callCount;
	int callCountPeriod;
	bool callCountActive;
	bool callDurationEnabled;
	int callDuration;
	int callDurationPeriod;
	bool callDurationActive;
	bool actionBlock;
	bool actionEmail;
	int applyToType;
	std::vector<std::string> prefixes;

	CallConstraintsRule() :
				id(0),
				callCountEnabled(false),
				callCount(0),
				callCountPeriod(0),
				callCountActive(false),
				callDurationEnabled(false),
				callDuration(0),
				callDurationPeriod(0),
				callDurationActive(false),
				actionBlock(false),
				actionEmail(false),
				applyToType(0)
	{
	}

	CallConstraintsRule(boost::property_tree::ptree &pt) {
		this->unserialize(pt);
	}
	void unserialize(boost::property_tree::ptree &pt);
	boost::property_tree::ptree serialize();
};

class CallPermissionsDisabledExtension : public AstDataStructureBase {
public:
	std::string username;
	std::string telNum;
	bool disabled;

	CallPermissionsDisabledExtension() :
				disabled(false)
	{
	}

	CallPermissionsDisabledExtension(boost::property_tree::ptree &pt) {
		this->unserialize(pt);
	}
	void unserialize(boost::property_tree::ptree &pt);
	boost::property_tree::ptree serialize();
};

class CallPermissionsEntry : public AstDataStructureBase {
public:
	int groupId;
	int policy;
	std::string prefix;
	std::string timeRangeName;
	std::string groupName;

	CallPermissionsEntry() :
				groupId(0),
				policy(0)
	{
	}

	CallPermissionsEntry(boost::property_tree::ptree &pt) {
		this->unserialize(pt);
	}
	void unserialize(boost::property_tree::ptree &pt);
	boost::property_tree::ptree serialize();
};

class CallPermissionsExtension : public AstDataStructureBase {
public:
	std::string telNum;
	int groupId;

	CallPermissionsExtension() :
				groupId(0)
	{
	}

	CallPermissionsExtension(boost::property_tree::ptree &pt) {
		this->unserialize(pt);
	}
	void unserialize(boost::property_tree::ptree &pt);
	boost::property_tree::ptree serialize();
};

class CallPermissionsInterface : public AstDataStructureBase {
public:
	std::string name;
	int groupId;

	CallPermissionsInterface() :
				groupId(0)
	{
	}

	CallPermissionsInterface(boost::property_tree::ptree &pt) {
		this->unserialize(pt);
	}
	void unserialize(boost::property_tree::ptree &pt);
	boost::property_tree::ptree serialize();
};

class DialplanExtension : public AstDataStructureBase {
public:
	std::string telNum;
	int type;

	DialplanExtension() :
				type(0)
	{
	}

	DialplanExtension(boost::property_tree::ptree &pt) {
		this->unserialize(pt);
	}
	void unserialize(boost::property_tree::ptree &pt);
	boost::property_tree::ptree serialize();
};

class TimeRangeEntry : public AstDataStructureBase {
public:
	int groupId;
	std::string groupName;
	int entryEnabled;
	std::string entryDescription;
	int entryType;
	char entryDays[8];
	int entryStart;
	int entryEnd;

	TimeRangeEntry() :
				groupId(0),
				entryEnabled(0),
				entryType(0),
				entryStart(0),
				entryEnd(0)
	{
				memset(entryDays, 0, sizeof(entryDays));
	}

	TimeRangeEntry(boost::property_tree::ptree &pt) {
		this->unserialize(pt);
	}
	void unserialize(boost::property_tree::ptree &pt);
	boost::property_tree::ptree serialize();
};

class RoutingTableItem : public AstDataStructureBase {
public:
	int id;
	std::string internal;
	std::string external;
	std::string iface;
	bool hideCallerId;

	RoutingTableItem() :
				id(0),
				hideCallerId(false)
	{
	}

	RoutingTableItem(boost::property_tree::ptree &pt) {
		this->unserialize(pt);
	}
	void unserialize(boost::property_tree::ptree &pt);
	boost::property_tree::ptree serialize();
};

#endif /* AST_DATA_STRUCTURES_H */
