#include <ActiveCallsInterface.h>

static struct ast_manager_event_blob *activecalls_calls_to_ami(struct stasis_message *message)
{
	struct ast_json_payload *payload = stasis_message_data(message);
	struct ast_manager_event_blob *res = NULL;

	struct ast_json *type = ast_json_object_get(payload->json, "type");
	switch (ast_json_integer_get(type)) {
		case 0:
			res = ast_manager_event_blob_create(EVENT_FLAG_SYSTEM, "KerioActiveCallsBegin", "%s", "");
			break;
		case 1:
		{
			struct ast_json *call = ast_json_object_get(payload->json, "call");
			char *str = ast_json_dump_string_format(call, AST_JSON_COMPACT);
			res = ast_manager_event_blob_create(EVENT_FLAG_SYSTEM, "KerioActiveCalls", "Call: %s\r\n", str);
			ast_json_free(str);
		}
			break;
		case 2:
			res = ast_manager_event_blob_create(EVENT_FLAG_SYSTEM, "KerioActiveCallsFinished", "%s", "");
			break;
	}

	return res;
}

STASIS_MESSAGE_TYPE_DEFN(activecalls_msg_type,
	.to_ami = activecalls_calls_to_ami,
	);

static char *activecalls_cli_show(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch (cmd) {
		case CLI_INIT:
			e->command = "kerio active_calls show";
			e->usage = "Usage: kerio active_calls show\n"
				"	Display active calls\n";
			return NULL;
		case CLI_GENERATE:
			return NULL;
		default:
			if (a->argc != e->args) {
				return CLI_SHOWUSAGE;
			}
	}

	activecalls_print(a->fd);
	return CLI_SUCCESS;
}

static char *activecalls_cli_ami(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch (cmd) {
		case CLI_INIT:
			e->command = "kerio active_calls ami";
			e->usage = "Usage: kerio active_calls ami\n"
				"	Send active calls over AMI\n";
			return NULL;
		case CLI_GENERATE:
			return NULL;
		default:
			if (a->argc != e->args) {
				return CLI_SHOWUSAGE;
			}
	}

	activecalls_amiSendCallsCb(0);
	return CLI_SUCCESS;
}


static struct ast_cli_entry activecalls_cli[] = {
	AST_CLI_DEFINE(activecalls_cli_show, "Display active calls"),
	AST_CLI_DEFINE(activecalls_cli_ami, "Send active calls over AMI"),
};

static int load_module_activecalls(void)
{
	STASIS_MESSAGE_TYPE_INIT(activecalls_msg_type);

	activecalls_load();

	ast_cel_backend_register("kerio_cel", activecalls_cel);
	ast_cli_register_multiple(activecalls_cli, ARRAY_LEN(activecalls_cli));

	return AST_MODULE_LOAD_SUCCESS;
}

static int unload_module_activecalls(void)
{
	ast_cel_backend_unregister("kerio_cel");
	ast_cli_unregister_multiple(activecalls_cli, ARRAY_LEN(activecalls_cli));

	activecalls_unload();

	STASIS_MESSAGE_TYPE_CLEANUP(activecalls_msg_type);

	return 0;
}

static int reload_module_activecalls(void)
{
	activecalls_reload();
	return 0;
}
