#include <CallConstraintsInterface.h>

static char *callconstraints_cli_show(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	int concise = 0;
	switch (cmd) {
		case CLI_INIT:
			e->command = "kerio call_constraints show [concise]";
			e->usage = "Usage: kerio call_constraints show [concise]\n"
				"	Display call constraints\n";
			return NULL;
		case CLI_GENERATE:
			return NULL;
		default:
			if (a->argc == e->args && (0 == strcmp(a->argv[3], "concise"))) {
				concise = 1;
			}
			else if (a->argc == e->args - 1) {
			}
			else {
				return CLI_SHOWUSAGE;
			}
	}

	callconstraints_print(a->fd, concise);
	return CLI_SUCCESS;
}

static struct ast_cli_entry callconstraints_cli[] = {
	AST_CLI_DEFINE(callconstraints_cli_show, "Display call constraints"),
};

/**
 * find extension in the callpermissions
 */
static int callconstraints_func_test_read(struct ast_channel *chan, const char *cmd, char *data, char *buf, size_t len)
{
	AST_DECLARE_APP_ARGS(args,
		AST_APP_ARG(dialedNumber);
		);

	if (ast_strlen_zero(data)) {
		/* empty argument - no number */
		return -1;
	}

	AST_STANDARD_APP_ARGS(args, data);

	if (args.argc != 1) {
		ast_log(AST_LOG_WARNING, "Syntax: KERIO_CALL_CONSTRAINTS(dialedNumber) - missing arguments!\n");
		return -1;
	}

	int status = callconstraints_test(args.dialedNumber);

	switch (status) {
		case 0: //miss
			snprintf(buf, len, "%d", 0);
			break;
		case 1: //hit
			snprintf(buf, len, "%d", 1);
			break;
		case -1: // invalid parameter
			snprintf(buf, len, "%d", 0);
			break;
	}

	return 0;
}

static struct ast_custom_function callconstraints_func_test = {
	.name = "KERIO_CALL_CONSTRAINTS",
	.desc = "Check call constraints before dialing a number",
	.syntax = "KERIO_CALL_CONSTRAINTS(dialedNumber)",
	.read = callconstraints_func_test_read,
};


static struct ast_sched_context *callconstraints_sched = NULL;

static int load_module_callconstraints(void)
{
	callconstraints_load();

	ast_cli_register_multiple(callconstraints_cli, ARRAY_LEN(callconstraints_cli));
	ast_custom_function_register(&callconstraints_func_test);

	/* scheduler */
	callconstraints_sched = ast_sched_context_create();
	if (!callconstraints_sched) {
		ast_log(AST_LOG_ERROR, "Failed to create ast_sched.\n");
		return AST_MODULE_LOAD_FAILURE;
	}
	if (ast_sched_start_thread(callconstraints_sched)) {
		ast_log(AST_LOG_ERROR, "Failed to create ast_sched_thread.\n");
		ast_sched_context_destroy(callconstraints_sched);
		return AST_MODULE_LOAD_FAILURE;
	}

	if (-1 == ast_sched_add(callconstraints_sched, 60 * 1000, callconstraints_sched_exec, NULL)) {
		ast_log(AST_LOG_ERROR, "Failed to schedule ast_sched.\n");
		ast_sched_context_destroy(callconstraints_sched);
		return AST_MODULE_LOAD_FAILURE;
	}

	return 0;
}

static int unload_module_callconstraints(void)
{
	ast_cli_unregister_multiple(callconstraints_cli, ARRAY_LEN(callconstraints_cli));
	ast_custom_function_unregister(&callconstraints_func_test);

	/* scheduler */
	if (callconstraints_sched) {
		ast_sched_context_destroy(callconstraints_sched);
		callconstraints_sched = NULL;
	}

	callconstraints_unload();

	return 0;
}

static int reload_module_callconstraints(void)
{
	callconstraints_unload();
	callconstraints_load();

	return 0;
}
