#include <CallPermissionsInterface.h>

static char *callpermissions_cli_show(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch (cmd) {
		case CLI_INIT:
			e->command = "kerio call_permissions show";
			e->usage = "Usage: kerio call_permissions show\n"
				"	Print loaded configuration\n";
			return NULL;
		case CLI_GENERATE:
			return NULL;
		default:
			if (a->argc != e->args) {
				return CLI_SHOWUSAGE;
			}
	}

	callpermissions_print(a->fd);
	return CLI_SUCCESS;
}

/**
 * find extension in the callpermissions
 */
static int callpermissions_func_test_read(struct ast_channel *chan, const char *cmd, char *data, char *buf, size_t len)
{
	AST_DECLARE_APP_ARGS(args,
		AST_APP_ARG(source);
		AST_APP_ARG(dialedNumber);
		);

	if (ast_strlen_zero(data)) {
		ast_log(AST_LOG_WARNING, "Syntax: KERIO_CALL_PERMISSIONS(source,dialedNumber) - missing arguments!\n");
		return -1;
	}

	AST_STANDARD_APP_ARGS(args, data);

	if (args.argc != 2) {
		ast_log(AST_LOG_WARNING, "Syntax: KERIO_CALL_PERMISSIONS(source,dialedNumber) - missing arguments!\n");
		return -1;
	}

	int status = callpermissions_test(args.source, args.dialedNumber);

	switch (status) {
		case 0: //miss
			snprintf(buf, len, "%d", 0);
			break;
		case 1: //hit
			snprintf(buf, len, "%d", 1);
			break;
		case -1: // invalid parameter
			snprintf(buf, len, "%d", 0);
			break;
	}

	return 0;
}

static struct ast_custom_function callpermissions_func_test = {
	.name = "KERIO_CALL_PERMISSIONS",
	.desc = "Check if extension exists in the callpermissions",
	.syntax = "KERIO_CALL_PERMISSIONS(source,dialedNumber)",
	.read = callpermissions_func_test_read,
};

static struct ast_cli_entry callpermissions_cli[] = {
	AST_CLI_DEFINE(callpermissions_cli_show, "Print loaded extensions"),
};

static int load_module_callpermissions(void)
{
	callpermissions_load();

	ast_cli_register_multiple(callpermissions_cli, ARRAY_LEN(callpermissions_cli));
	ast_custom_function_register(&callpermissions_func_test);

	return AST_MODULE_LOAD_SUCCESS;
}

static int unload_module_callpermissions(void)
{
	ast_cli_unregister_multiple(callpermissions_cli, ARRAY_LEN(callpermissions_cli));
	ast_custom_function_unregister(&callpermissions_func_test);

	callpermissions_unload();

	return 0;
}

static int reload_module_callpermissions(void)
{
	callpermissions_unload();
	callpermissions_load();

	return 0;
}
