#include <module.h>
#include <TimeRangeInterface.h>

static char *timerage_cli_show(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch (cmd) {
		case CLI_INIT:
			e->command = "kerio time_range show";
			e->usage = "Usage: kerio time_range show\n"
				"	Print loaded extensions\n";
			return NULL;
		case CLI_GENERATE:
			return NULL;
		default:
			if (a->argc != e->args) {
				return CLI_SHOWUSAGE;
			}
	}

	timerange_print(a->fd);
	return CLI_SUCCESS;
}

static char *timerage_cli_test(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
{
	switch (cmd) {
		case CLI_INIT:
			e->command = "kerio time_range test";
			e->usage = "Usage: kerio time_range test <time range>\n"
				"	Test current time against a time range\n";
			return NULL;
		case CLI_GENERATE:
			if (a->pos == 3) {
				char **choices = timerange_get_names();
				char *choice = ast_cli_complete(a->word, (const char**) choices, a->n);

				int i = 0;
				for (i = 0; choices[i]; i++) {
					free(choices[i]);
					choices[i] = NULL;
				}
				free(choices);

				return choice;
			}
			return NULL;

		default:
			if (a->argc != e->args + 1) {
				return CLI_SHOWUSAGE;
			}
	}

	ast_cli(a->fd, "Testing current time against time range \"%s\": ", a->argv[3]);
	switch (timerange_test(a->argv[3])) {
		case 0:
			ast_cli(a->fd, "miss!\n");
			break;
		case 1:
			ast_cli(a->fd, "hit!\n");
			break;
		case -1:
			ast_cli(a->fd, "time range not found!\n");
			break;
	}

	return CLI_SUCCESS;
}

/**
 * find extension in the timerange
 */
static int timerange_func_find_read(struct ast_channel *chan, const char *cmd, char *data, char *buf, size_t len)
{
	AST_DECLARE_APP_ARGS(args,
		AST_APP_ARG(timeRange);
		);

	if (ast_strlen_zero(data)) {
		ast_log(AST_LOG_WARNING, "Syntax: KERIO_TIME_RANGE(timeRange) - missing arguments!\n");
		return -1;
	}

	AST_STANDARD_APP_ARGS(args, data);

	if (args.argc != 1) {
		ast_log(AST_LOG_WARNING, "Syntax: KERIO_TIME_RANGE(timeRange) - missing arguments!\n");
		return -1;
	}

	switch (timerange_test(args.timeRange)) {
		case 0: //miss
			ast_verb(3, "Time range %s... [FALSE]\n", args.timeRange);
			snprintf(buf, len, "%d", 0);
			break;
		case 1: //hit
			ast_verb(3, "Time range %s... [TRUE]\n", args.timeRange);
			snprintf(buf, len, "%d", 1);
			break;
		case -1: // invalid parameter
			ast_log(AST_LOG_WARNING, "Time range %s... [NOT FOUND]\n", args.timeRange);
			snprintf(buf, len, "%d", 0);
			break;
	}

	return 0;
}

static struct ast_custom_function timerange_func_find = {
	.name = "KERIO_TIME_RANGE",
	.desc = "Check if current time is within in a time range",
	.syntax = "KERIO_TIME_RANGE(timeRange)",
	.read = timerange_func_find_read,
};

static struct ast_cli_entry timerange_cli[] = {
	AST_CLI_DEFINE(timerage_cli_show, "Print loaded timeranges"),
	AST_CLI_DEFINE(timerage_cli_test, "Test current time against a time range"),
};

static int load_module_timerange(void)
{
	timerange_load();

	ast_cli_register_multiple(timerange_cli, ARRAY_LEN(timerange_cli));
	ast_custom_function_register(&timerange_func_find);

	return AST_MODULE_LOAD_SUCCESS;
}

static int unload_module_timerange(void)
{
	ast_cli_unregister_multiple(timerange_cli, ARRAY_LEN(timerange_cli));
	ast_custom_function_unregister(&timerange_func_find);

	timerange_unload();

	return 0;
}

static int reload_module_timerange(void)
{
	timerange_unload();
	timerange_load();

	return 0;
}
