#include "Blacklist.h"

#define CONFIG_FILE "/etc/asterisk/kerio_blacklist.conf"

void Blacklist::load()
{
	if (!fileChanged(CONFIG_FILE, this->lastTimestamp)) {
		return; /* no change */
	}
	blacklistCache.clear();

	/* load data */
	FILE *fp = fopen(CONFIG_FILE, "r");
	if (!fp) {
		ast_log(AST_LOG_ERROR, "Failed to open file %s\n", CONFIG_FILE);
		return;
	}

	while (!feof(fp)) {
		char tmpBuff[1024];
		if (fgets(tmpBuff, sizeof (tmpBuff), fp)) {
			BlacklistCacheItem bic;
			for (char *c = tmpBuff; *c != '\n' && *c != 0; c++) {
				switch (*c) {
					case 'x':
					case 'X':
						/* one character */
						bic.expression += '.';
						break;
					case '.':
						/* zero or more characters */
						bic.expression += ".*";
						break;
					case '*':
					case '+':
					case '#':
						/* escape special chars */
						bic.expression += "\\";
						bic.expression.append(c, 1);
						break;
					default:
						bic.expression.append(c, 1);
				}
			}
			try {
				bic.re.assign(bic.expression, boost::regex::icase);
			}
			catch (boost::regex_error& e) {
				ast_log(AST_LOG_ERROR, "Failed to load blacklist entry %s:%s\n", bic.expression.c_str(), e.what());
				continue;
			}

			blacklistCache.push_back(bic);
		}
	}
	fclose(fp);

	ast_verb(3, "Loaded %zu valid items.\n", blacklistCache.size());
}

Blacklist::Blacklist() : lastTimestamp(0)
{
	load();
}

Blacklist::~Blacklist() { }

void Blacklist::print(const int fd)
{
	load();
	ast_cli(fd, "Blacklist\n");
	ast_cli(fd, "----------\n");

	BOOST_FOREACH(BlacklistCacheItem item, blacklistCache)
	{
		ast_cli(fd, "%s\n", item.expression.c_str());
	}
}

/**
 * Match number against the blacklist
 *
 * @param number [std::string] - extension to look for
 *
 * @return [int] - 1 found, 0 not found
 */
int Blacklist::match(const std::string &number)
{
	load();

	ast_verb(3, "Searching for number '%s'...\n", number.c_str());

	BOOST_FOREACH(BlacklistCacheItem item, blacklistCache)
	{
		if (boost::regex_match(number, item.re)) {
			ast_verb(3, "Searching for number '%s'... FOUND(%s)\n", number.c_str(), item.expression.c_str());
			return 1;
		}
	}

	ast_verb(3, "Searching for number '%s'... NOT FOUND\n", number.c_str());
	return 0;
}
