#include "Interface.h"
#include "Routing.h"
#include "RoutingInterface.h"

static Routing *routing;
static Lock loadLock;

int routing_load()
{
	LockUnique l(&loadLock.l);
	if (!l.ok) return 0;

	routing = new Routing();
	return 0;
}

void routing_unload()
{
	LockUnique l(&loadLock.l);
	if (!l.ok) return;

	if (routing) {
		delete routing;
		routing = NULL;
	}
}

void routing_find_incoming(char *buf, size_t len, const char *interface, const char *number)
{
	LockShared l(&loadLock.l);
	if (!routing) {
		return;
	}

	std::string ret = routing->find_incoming(interface, number);
	snprintf(buf, len, "%s", ret.c_str());
}

void routing_find_incoming_rev(char *buf, size_t len, const char *interface, const char *number)
{
	LockShared l(&loadLock.l);
	if (!routing) {
		return;
	}

	std::string ret = routing->find_incoming_rev(interface, number);
	snprintf(buf, len, "%s", ret.c_str());
}

void routing_find_outgoing(char *buf, size_t len, const char *routeId, const char *number)
{
	LockShared l(&loadLock.l);
	if (!routing) {
		return;
	}

	std::string ret = routing->find_outgoing(routeId, number);
	snprintf(buf, len, "%s", ret.c_str());
}


void routing_hidden(char *buf, size_t len, const char *routeId, const char *number)
{
	LockShared l(&loadLock.l);
	if (!routing) {
		return;
	}

	bool ret = routing->is_hidden(routeId, number);
	snprintf(buf, len, "%d", ret ? 1 : 0);
}

void routing_print(const int fd)
{
	LockShared l(&loadLock.l);
	if (!routing) {
		return;
	}
	routing->print(fd);
}
