#include "Interface.h"
#include "Security.h"
#include "SecurityInterface.h"

static Security *security;
static Lock loadLock;

int security_load()
{
	LockUnique l(&loadLock.l); if (!l.ok) return 0;

	security = new Security();
	return 0;
}

void security_unload()
{
	LockUnique l(&loadLock.l); if (!l.ok) return;

	if (security) {
		delete security;
		security = NULL;
	}
}

void security_print(const int fd)
{
	LockShared l(&loadLock.l);
	if (!security) {
		return;
	}

	security->print(fd);
}

extern "C" {

	int resKerioSecurityCheck(const struct sockaddr_in *sin, const char *user)
	{
		LockShared l(&loadLock.l);
		if (!security) {
			return 0;
		}

		return security->authCheck(sin, user);
	}

	void resKerioSecurityAuthFailure(const struct sockaddr_in *sin, const char *user)
	{
		LockShared l(&loadLock.l);
		if (!security) {
			return;
		}

		security->authFailure(sin, user);
	}

	void resKerioSecurityAuthSuccess(const struct sockaddr_in *sin, const char *user)
	{
		LockShared l(&loadLock.l);
		if (!security) {
			return;
		}

		security->authSuccess(sin, user);
	}
}
